{{
*****************************
* Debug kernel for assembly *
* code on-chip debugging    *
*  (C) 2007 Thomas Kliche   *
*****************************

Revision History
----------------
 1.1 on 29 March 2007
     - fixed command 0 problem
     - renamed entry point label to prevent that debug kernel does not start
     - renamed method start and stop to startDebug and stopDebug and variable
       Cog to DebugCog to prevent name clash when adding code to debug
     - added method initDebug with separated code to initialize debug kernel
     - added parameter on method startDebug to pass parameter to assembly code

 1.0 on 18 March 2007
     - first public version

...................................................................................................

This is a debug kernel which provides some generic functionality that you can use
to debug your assembly code running in a Cog.

---------------------------------------------------------------------------------------------------
}}
CON

  FLAG_C          = %01                                 'bit position for C flag
  FLAG_NZ         = %10                                 'bit position for inverted Z flag

  'commands for communication with debug kernel
  CMD_RDMAINDATA  = 1                                   'read data from main memory
  CMD_WRMAINDATA  = 2                                   'write data to main memory
  CMD_RDCOGDATA   = 3                                   'read data from Cog memory
  CMD_WRCOGDATA   = 4                                   'write data to Cog memory
  CMD_RDFLAGS     = 5                                   'read flags


VAR

  long  DebugCog


PUB initDebug(dbgParamPtr)

  LONG[dbgParamPtr][-2] := _DbgParam
  LONG[dbgParamPtr][-1] := _DbgParamPtr
  LONG[dbgParamPtr][1]  := @debugEntry
  LONG[dbgParamPtr][2]  := _DbgTemp
  _DbgParam := dbgParamPtr


PUB startDebug(yourParamPtr)

  stopDebug
  DebugCog := cognew(@debugEntry, yourParamPtr) + 1


PUB stopDebug

  if DebugCog
    cogstop(DebugCog~ - 1)


DAT
''
''The debug kernel requires 36 longs of Cog memory including the jmp statement to give control to
''the debugger.
''
                        org     0
debugEntry              jmp     #debugger               'start debugger
'
'PLACE YOUR CODE TO DEBUG HERE EXCEPT THE UNINITIALIZED VARIABLES!
'PLACE YOUR UNINITIALIZED VARIABLES AT THE END OF THIS FILE!
'I RECOMMEND TO PLACE ALL YOUR VARIABLES AT THE END OF THIS FILE EXCEPT THOSE ARE EVENTUALLY
'LOCATED BETWEEN ASSEMBLY INSTRUCTIONS!
'
                        'jmpret  _DbgTemp,#getData2      'instruction used as breakpoint

'==================================================================================================
{
Code of kernel to debug your assembly program. The debug kernel requires 36 longs of Cog memory
including a jmp statement to give control to the debugger.
}
getData                 movs    :readData,_DbgAddr
                        nop
:readData               mov     _DbgTemp,0-0
getData2                wrlong  _DbgTemp,_DbgParamPtr

'-----------------------
debugger                mov     _DbgTemp,#0
                        wrlong  _DbgTemp,_DbgParam      'send ready
                        mov     _DbgParamPtr,_DbgParam
                        add     _DbgParamPtr,#4         'point to DebugAddr
:dbgWaitCmd             rdlong  _DbgTemp,_DbgParam
                        tjz     _DbgTemp,#:dbgWaitCmd
                        rdlong  _DbgAddr,_DbgParamPtr
                        add     _DbgTemp,#dbgCmdTable - 1
                        movs    :dbgJmpCmd,_DbgTemp
                        add     _DbgParamPtr,#4         'point to DebugData
:dbgJmpCmd              jmp     #0-0                    'this is also used to run the debugged code
dbgCmdTable             jmp     #getMainData
                        jmp     #setMainData
                        jmp     #getData
                        jmp     #setData

'-----------------------
getFlags                muxnz   _DbgTemp,#FLAG_NZ
                        muxc    _DbgTemp,#FLAG_C
                        jmp     #getData2

'-----------------------
setData                 movd    :writeData,_DbgAddr
                        rdlong  _DbgTemp,_DbgParamPtr
:writeData              mov     0-0,_DbgTemp
                        jmp     #debugger

'-----------------------
getMainData             rdlong  _DbgTemp,_DbgAddr
                        jmp     #getData2

'-----------------------
setMainData             rdlong  _DbgTemp,_DbgParamPtr
                        wrlong  _DbgTemp,_DbgAddr
                        jmp     #debugger

'#######################################################
'variables used by debugger
_DbgParam               jmpret  _DbgTemp,#getData2      'this points always to the first parameter like register par
_DbgParamPtr            long    dbgCmdTable - 1         'pointer to access parameters
_DbgAddr                long    0                       'debugged address
_DbgTemp                long    _DbgVariables
_DbgVariables
'==================================================================================================
'
'PLACE YOUR UNINITIALIZED VARIABLES HERE!
'